﻿// 
// This is the source code of Telegram for Windows Phone v. 3.x.x.
// It is licensed under GNU GPL v. 2 or later.
// You should have received a copy of the license in this archive (see LICENSE).
// 
// Copyright Evgeny Nadymov, 2013-present.
// 
using System.IO;
using System.Linq;
using Telegram.Api.Extensions;

namespace Telegram.Api.TL
{
    public class TLHashtagItem : TLObject
    {
        public const uint Signature = TLConstructors.TLHashtagItem;

        public TLString Hashtag { get; set; }

        public TLHashtagItem()
        {
            
        }

        public TLHashtagItem(string hashtag)
        {
            Hashtag = new TLString(hashtag);
        }

        public override TLObject FromBytes(byte[] bytes, ref int position)
        {
            bytes.ThrowExceptionIfIncorrect(ref position, Signature);

            Hashtag = GetObject<TLString>(bytes, ref position);

            return this;
        }

        public override byte[] ToBytes()
        {
            return TLUtils.Combine(
                TLUtils.SignatureToBytes(Signature),
                Hashtag.ToBytes());
        }

        public override TLObject FromStream(Stream input)
        {
            Hashtag = GetObject<TLString>(input);

            return this;
        }

        public override void ToStream(Stream output)
        {
            output.Write(TLUtils.SignatureToBytes(Signature));
            output.Write(Hashtag.ToBytes());
        }

        public override string ToString()
        {
            return Hashtag != null ? Hashtag.ToString() : string.Empty;
        }
    }
}
